@@ -770,9 +770,8 @@ exchange_mapi_util_get_gal (GPtrArray *contacts_array)
 	ulFlags = TABLE_START;
 	do {
 		count += 0x2;
-		SRowSet = NULL;
 		retval = GetGALTable(global_mapi_session, SPropTagArray, &SRowSet, count, ulFlags);
-		if ((!SRowSet) || (!(SRowSet->aRow)) || retval != MAPI_E_SUCCESS) {
+		if ((!SRowSet) || (!(SRowSet->aRow))) {
 			UNLOCK ();
 			MAPIFreeBuffer (SPropTagArray);
 			return FALSE;
@@ -945,10 +944,7 @@ exchange_mapi_util_modify_recipients (TALLOC_CTX *mem_ctx, mapi_object_t *obj_me
 			set_recipient_properties(mem_ctx, &SRowSet->aRow[last], recipient, TRUE);
 			SRowSet->cRows += 1;
 		} else if (FlagList->aulPropTag[i] == MAPI_RESOLVED) {
-			/* FIXME: Even though some recipients are resolved, their mailboxes need not reside 
-			on the same server in case of cluster setups. So lets set ext_lpProps for all recipients.
-			Lets have this fix until we have openchange handling this case better. */
-			set_recipient_properties (mem_ctx, &SRowSet->aRow[j], recipient, TRUE);
+			set_recipient_properties (mem_ctx, &SRowSet->aRow[j], recipient, FALSE);
 			j += 1;
 		}
 	}
@@ -1308,6 +1304,10 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 				cb_retval = cb (item_data, data);
 
 				g_free (item_data);
+			} else {
+				exchange_mapi_util_free_stream_list (&stream_list);
+				exchange_mapi_util_free_recipient_list (&recip_list);
+				exchange_mapi_util_free_attachment_list (&attach_list);
 			}
 
 			if (GetPropsTagArray->cValues) 
@@ -1486,6 +1486,10 @@ exchange_mapi_connection_fetch_item (mapi_id_t fid, mapi_id_t mid,
 		cb (item_data, data);
 
 		g_free (item_data);
+	} else {
+		exchange_mapi_util_free_stream_list (&stream_list);
+		exchange_mapi_util_free_recipient_list (&recip_list);
+		exchange_mapi_util_free_attachment_list (&attach_list);
 	}
 
 //	if (GetPropsTagArray->cValues) 
@@ -2151,22 +2155,6 @@ exchange_mapi_create_item (uint32_t olFolder, mapi_id_t fid,
 		retval = SubmitMessage(&obj_message);
 		if (retval != MAPI_E_SUCCESS) {
 			mapi_errstr("SubmitMessage", GetLastError());
-
-			if ((options & MAPI_OPTIONS_DELETE_ON_SUBMIT_FAILURE) != 0) {
-				mid = mapi_object_get_id (&obj_message);
-				mapi_object_release(&obj_message);
-				/* to not release a message object twice */
-				mapi_object_init (&obj_message);
-
-				retval = DeleteMessage (&obj_folder, &mid, 1);
-				if (retval != MAPI_E_SUCCESS) {
-					mapi_errstr ("DeleteMessage", GetLastError ());
-				}
-
-				/* do not forget to set it back to 0, as the function failed */
-				mid = 0;
-			}
-
 			goto cleanup;
 		}
 	}
@@ -2709,10 +2697,11 @@ mapi_get_ren_additional_fids (mapi_object_t *obj_store, GHashTable **folder_list
 	/* Iterate through MV_BINARY */
 	if (entryids) {
 		for (i = 0; i < G_N_ELEMENTS (olfolder_defaults); i++) {
+			fid = 0;
 			entryid = entryids->lpbin [i];
 			retval = GetFIDFromEntryID(entryid.cb, entryid.lpb, inbox_id, &fid);
 
-			if (fid) {
+			if (retval == MAPI_E_SUCCESS && fid) {
 				folder_type = g_new0 (guint32, 1);
 				*folder_type = olfolder_defaults[i];
 
